/*
 * minios_boot - Boot Bitvisor with TRESOR config from Linux
 *
 * Copyright (C) 2013 Johannes Goetzfried <johannes@jgoetzfried.de>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307 USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "boot.h"

static void load_random_seed(struct config_data *cfg, char *devname)
{
	int r;
	FILE *fp;

	fp = fopen (devname, "r");
	if (!fp) {
		perror ("urandom");
		exit (EXIT_FAILURE);
	}

	r = fread(cfg->vmm.randomSeed, sizeof cfg->vmm.randomSeed, 1, fp);
	if (r != 1) {
		perror ("urandom");
		exit (EXIT_FAILURE);
	}
	fclose (fp);
}


int main() {
	size_t ret;

	memset(&config, 0, sizeof config);

	memset(&config.storage.keys_conf[0].guid.data1, 0xff, sizeof(struct guid)); // STORAGE_GUID_ANY
	config.storage.keys_conf[0].type      = STORAGE_TYPE_ANY;
	config.storage.keys_conf[0].host_id   = STORAGE_HOST_ID_ANY;
	config.storage.keys_conf[0].device_id = STORAGE_DEVICE_ID_ANY;

	config.storage.keys_conf[0].lba_low   = 0x0ULL;
	config.storage.keys_conf[0].lba_high  = 0xffffffffffffffffULL;

	strncpy(&config.storage.keys_conf[0].crypto_name[0], "tresor", 8);
	config.storage.keys_conf[0].keyindex  = 0;
	config.storage.keys_conf[0].keybits   = 256;

	config.vmm.auto_reboot = 0;
	config.vmm.driver.ata  = 1;


	/* read the dek from stdin */
	ret = fread(&config.storage.keys[0][0], 32, 1, stdin);
	if (ret != 1) {
		fprintf(stderr, "Could not read DEK!\n");
		return EXIT_FAILURE;
	}


	load_random_seed(&config, "/dev/urandom");

	printf("Booting into the main system...\n");

	boot_guest();

	return EXIT_SUCCESS;
}
