/*
 * AES XTS encryption routines in userspace
 *
 * Copyright (C) 2013 Johannes Goetzfried <johannes@jgoetzfried.de>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307 USA.
 */

#ifndef _AES_XTS_H
#define _AES_XTS_H

#define AES_BLK_BYTES   16

#include <openssl/aes.h>


struct aes_xts_encrypt_keys {
	AES_KEY	tweak_key;
	AES_KEY	encrypt_key;
};
struct aes_xts_decrypt_keys {
	AES_KEY	tweak_key;
	AES_KEY	decrypt_key;
};
struct aes_xts_keyctx {
	struct aes_xts_encrypt_keys encrypt;
	struct aes_xts_decrypt_keys decrypt;
};

typedef void (*aes_crypt_func_t)(const uint8_t *in, uint8_t *out, const AES_KEY *key);


void aes_xts_encrypt(uint8_t *dst, const uint8_t *src,
		     const struct aes_xts_keyctx *k, uint64_t lba, uint32_t sector_size);

void aes_xts_decrypt(uint8_t *dst, const uint8_t *src,
		     const struct aes_xts_keyctx *k, uint64_t lba, uint32_t sector_size);

struct aes_xts_keyctx *aes_xts_setkey(const uint8_t *key, int bits);

void free_aes_xts_keyctx(struct aes_xts_keyctx *keyctx);

#endif /* _AES_XTS_H */
