# MARK - Communication with the Teensy USB Device
#
# Copyright (C) 2013  Johannes Goetzfried <johannes@jgoetzfried.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import config;
import time;
import subprocess;

from serial import Serial;

class TeensyError(Exception):
	def __init__(self, value):
		self.value = value;
	def __str__(self):
		return repr(self.value);

class Teensy:
	def __init__(self, device):
		self._ser = Serial(device, 115200, timeout=1);
		self._ser.readline();

	def verify(self, nonce):
		self._ser.write('verify ' + nonce + '\n');
		ret = self._ser.readline();
		if ret[0] != '2':
			raise TeensyError(ret[:-2]);

	def store(self, nonce):
		self._ser.write('store ' + nonce + '\n');
		ret = self._ser.readline();
		if ret[0] != '2':
			raise TeensyError(ret[:-2]);

	def close(self):
		self._ser.close();


def flash():
	cmd = 'dfu-programmer ' + config.teensy_mcu + ' ';
	tail = ' > /dev/null 2>&1';
	subprocess.check_call(cmd + 'erase' + tail, shell=True);
	subprocess.check_call(cmd + 'flash ' + config.teensy_flash + tail, shell=True);
	subprocess.check_call(cmd + 'flash-eeprom ' + config.teensy_eeprom + tail, shell=True);

def start():
	cmd = 'dfu-programmer ' + config.teensy_mcu + ' ';
	subprocess.check_call(cmd + 'start > /dev/null 2>&1', shell=True);
	time.sleep(4);

